#ifndef ANTGO_INCLUDE_CTYPES_H_
#define ANTGO_INCLUDE_CTYPES_H_

namespace antgo {

typedef wchar_t wchar;
typedef char byte;
typedef unsigned char ubyte;
typedef unsigned short ushort;
typedef unsigned int uint;
typedef unsigned long ulong;
typedef long long longlong;
typedef unsigned long long ulonglong;
typedef long double longdouble;
typedef char* char_p;
typedef wchar_t* wchar_p;
typedef void* void_p;

}  // namespace antgo

#endif  // ANTGO_INCLUDE_CTYPES_H_
