#ifndef ANTGO_INCLUDE_CONTEXT_CONTEXT_H_
#define ANTGO_INCLUDE_CONTEXT_CONTEXT_H_

#include <cstring>

#include "./api.h"

#define ANTGO_FUNC
// #if USING_HIP || USING_CUDA
// #include "./hip_ctx.h"
// #else
// #include "./cpu_ctx.h"
// #endif  // USING_CUDA

// extern "C" {
// ANTGO_API void set_device(const int device_id);
// }

#endif  // ANTGO_INCLUDE_CONTEXT_CONTEXT_H_
