/*
 * ANTGOOP Wrapper generated from the source code ${cpp_fname}
 * Created by: ANTGOOP ${git_hash}
 * Create Time: ${create_time}
 *
 * WARNING! All changes made in this file will be lost!
 */
# include "defines.h"
#include "${inc_fname}"

extern "C" {

${code}
}
